/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "docwindow.h"

#include "mandelbrot.h"
#include "feigenbaum.h"
#include "drawwidget.h"
#include "icons.h"

#include <qapplication.h>
#include <qscrollview.h>
#include <qstatusbar.h>
#include <qlabel.h>

DocWindow::DocWindow( DocInterface* doc, QWidget* parent )
	: QMainWindow( parent, 0, WDestructiveClose ), m_doc( doc ),
	  m_mouseutility( zoom ), m_zooming( false ), m_moving( false )
{
	connect( m_doc, SIGNAL( resized() ), SLOT( docResized() ) );
	connect( m_doc, SIGNAL( rectPainted( const QRect& ) ), SLOT( refresh( const QRect& ) ) );
	connect( m_doc, SIGNAL( startedCalc() ), SLOT( startClock() ) );
	connect( m_doc, SIGNAL( stoppedCalc() ), SLOT( stopClock() ) );

	m_scrollview = new QScrollView( this );
	setCentralWidget( m_scrollview );

	m_docwidget = new DrawWidget( m_scrollview );
	connect( m_docwidget, SIGNAL( paint( const QRect& ) ), SLOT( refresh( const QRect& ) ) );
	connect( m_docwidget, SIGNAL( mouseDownLeft( const QPoint& ) ), SLOT( mouseDownLeft( const QPoint& ) ) );
	connect( m_docwidget, SIGNAL( mouseUpLeft( const QPoint& ) ), SLOT( mouseUpLeft( const QPoint& ) ) );
	connect( m_docwidget, SIGNAL( mouseDownRight( const QPoint& ) ), SLOT( mouseDownRight( const QPoint& ) ) );
	connect( m_docwidget, SIGNAL( mouseMove( const QPoint& ) ), SLOT( mouseMove( const QPoint& ) ) );
	connect( m_docwidget, SIGNAL( mouseLeave() ), SLOT( mouseLeave() ) );
	m_docwidget->setFixedSize( m_doc->picSize().width, m_doc->picSize().height );

	m_scrollview->addChild( m_docwidget, 0, 0 );

	m_status_msg = new QLabel( "Bereit.", statusBar() );
	statusBar()->addWidget( m_status_msg, 1 );
	m_status_coords = new QLabel( statusBar() );
	statusBar()->addWidget( m_status_coords, 1 );
	statusBar()->setSizeGripEnabled( false );

	if( m_doc->isA( "Mandelbrot::Document" ) )
	{
		setIcon( QPixmap( mb_icon_xpm ) );
		m_docwidget->setBackgroundColor( Qt::black );
	}
	else if( m_doc->isA( "Feigenbaum::Document" ) )
	{
		setIcon( QPixmap( fb_icon_xpm ) );
		m_docwidget->setBackgroundColor( Qt::white );
	}

}

DocWindow::~DocWindow()
{
	if( m_doc )
		delete m_doc;
	m_doc = 0;
}

DocInterface* DocWindow::doc()
{
	return m_doc;
}

DocWindow::MouseUtility DocWindow::mouseUtility()
{
	return m_mouseutility;
}

void DocWindow::setMouseUtility( MouseUtility new_mouseutility )
{
	m_mouseutility = new_mouseutility;
}

void DocWindow::docResized()
{
	m_docwidget->setFixedSize( m_doc->picSize().width, m_doc->picSize().height );
	refresh( m_docwidget->rect() );
}

void DocWindow::refresh( const QRect& rect )
{
	if( m_doc )
	{
		if( ! m_doc->pixmap().isNull() )
			bitBlt( m_docwidget, rect.topLeft(), &m_doc->pixmap(), rect );
		else
			m_docwidget->erase();
	}
}

void DocWindow::mouseDownLeft( const QPoint& point )
{
	m_rect_mouse.tl.x = m_rect_mouse.br.x = point.x();
	m_rect_mouse.tl.y = m_rect_mouse.br.y = point.y();

	if( ! m_doc->calcing() )
	{
		m_zooming = m_mouseutility == zoom;
		m_moving = m_mouseutility == move;
	}

}

void DocWindow::mouseUpLeft( const QPoint& point )
{
	IntPoint point_mouseup( point.x(), point.y() );

	if( ! m_doc ) return;
	if( m_doc->calcing() ) return;

	switch( m_mouseutility )
	{
		case zoom:
			if( ! m_zooming ) return;

			drawZoomRect();
			m_zooming = false;

			m_rect_mouse.br = point_mouseup;
			if( m_rect_mouse.width() != 0 && m_rect_mouse.height() != 0 )
			{
				if( m_rect_mouse.tl.x > m_rect_mouse.br.x )
				{
					int tmp = m_rect_mouse.tl.x;
					m_rect_mouse.tl.x = m_rect_mouse.br.x;
					m_rect_mouse.br.x = tmp;
				}
				if( m_rect_mouse.tl.y > m_rect_mouse.br.y )
				{
					int tmp = m_rect_mouse.tl.y;
					m_rect_mouse.tl.y = m_rect_mouse.br.y;
					m_rect_mouse.br.y = tmp;
				}

				double ratio = (double) m_doc->picSize().width / m_doc->picSize().height;
				double zoom_ratio = (double) m_rect_mouse.width() / m_rect_mouse.height();
				
				if( zoom_ratio >= ratio )
					m_rect_mouse.br.y = m_rect_mouse.tl.y + 1 / ratio * m_rect_mouse.width();
				else
					m_rect_mouse.br.x = m_rect_mouse.tl.x + ratio * m_rect_mouse.height();

				DoubleRect new_coords( translatePoint( m_rect_mouse.tl ), translatePoint( m_rect_mouse.br ) );
				m_doc->setCoords( new_coords );
			}
			else
			{
				qApp->beep();
			}
			break;
		case center:
			if( m_rect_mouse.tl == point_mouseup )
			{
				DoublePoint coords_point = translatePoint( point_mouseup );
				DoubleRect new_coords( coords_point.x - m_doc->coords().width() / 2, coords_point.y - m_doc->coords().height() / 2, coords_point.x + m_doc->coords().width() / 2, coords_point.y + m_doc->coords().height() / 2 );
				m_doc->setCoords( new_coords );
			}
			else
			{
				qApp->beep();
			}
			break;
		case move:
			if( ! m_moving )
				return;

			m_moving = false;

			DoubleRect coords = m_doc->coords();
			double dx = coords.width() / m_doc->picSize().width * ( point_mouseup.x - m_rect_mouse.tl.x );
			double dy = coords.height() / m_doc->picSize().height * ( point_mouseup.y - m_rect_mouse.tl.y );

			coords.tl.x -= dx;
			coords.br.x -= dx;
			coords.tl.y -= dy;
			coords.br.y -= dy;

			m_doc->setCoords( coords );
			break;
	}
}

void DocWindow::mouseDownRight( const QPoint& point )
{
	if( m_mouseutility == zoom && m_zooming )
	{
		drawZoomRect();
		m_zooming = false;
		qApp->beep();
	}

	if( m_mouseutility == move && m_moving )
	{
		m_moving = false;
		refresh( m_doc->pixmap().rect() );
		qApp->beep();
	}
}

void DocWindow::mouseMove( const QPoint& point )
{
	DoublePoint trans_point = translatePoint( IntPoint( point.x(), point.y() ) );
	m_status_coords->setText( QString( "x: %1 y: %2" ).arg( trans_point.x ).arg( trans_point.y ) );

	if( m_zooming )
	{
		drawZoomRect();
		m_rect_mouse.br.x = point.x();
		m_rect_mouse.br.y = point.y();
		drawZoomRect();
	}

	if( m_moving )
	{
		if( ! m_doc->pixmap().isNull() )
		{
			QPixmap& pixmap = m_doc->pixmap();

			QRegion regdoc( m_docwidget->rect() );
			QRegion regbitmap( point.x() - m_rect_mouse.tl.x, point.y() - m_rect_mouse.tl.y, pixmap.width(), pixmap.height() );

			m_docwidget->erase( regdoc - regbitmap );
			bitBlt( m_docwidget, point.x() - m_rect_mouse.tl.x, point.y() - m_rect_mouse.tl.y, &pixmap );
		}
	}
}

void DocWindow::mouseLeave()
{
	m_status_coords->setText( "" );
}

void DocWindow::startClock()
{
	m_status_msg->setText( "Berechne..." );
	m_time.start();
}

void DocWindow::stopClock()
{
	int ms = m_time.elapsed();
	int days, hours, minutes, seconds;
	ms -= ( days = ms / ( 24*60*60*1000 ) ) * 24*60*60*1000;
	ms -= ( hours = ms / ( 60*60*1000 ) ) * 60*60*1000;
	ms -= ( minutes = ms / ( 60*1000 ) ) * 60*1000;
	ms -= ( seconds = ms / 1000 ) * 1000;

	QString msg = "Rechendauer:";
	if( days > 0 )
		msg = ( msg += " %1d" ).arg( days );
	if( hours > 0 )
		msg = ( msg += " %1h" ).arg( hours );
	if( minutes > 0 )
		msg = ( msg += " %1min" ).arg( minutes );
	if( seconds > 0 )
		msg = ( msg += " %1sec" ).arg( seconds );
	if( ms > 0 )
		msg = ( msg += " %1ms" ).arg( ms );

	m_status_msg->setText( msg );
}

DoublePoint DocWindow::translatePoint( const IntPoint& point )
{
	double x = m_doc->coords().tl.x + (double) point.x / m_doc->picSize().width * m_doc->coords().width();
	double y = m_doc->coords().tl.y + (double) point.y / m_doc->picSize().height * m_doc->coords().height();
	return DoublePoint( x, y );
}

void DocWindow::drawZoomRect()
{
	QPainter painter( m_docwidget );
	painter.setPen( Qt::DotLine );
	painter.setRasterOp( Qt::NotXorROP );
	painter.drawRect( m_rect_mouse.tl.x, m_rect_mouse.tl.y, m_rect_mouse.width(), m_rect_mouse.height() );
}
