/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "feigenbaum.h"

#include <qfileinfo.h>

Feigenbaum::Document::Document()
	: DocInterface()
{
	m_input = new CalcInputData();
	m_result = new CalcResult();
	m_calc = new Calc( m_input, m_result );
	m_fileaccess = new FileAccess();
	m_actioncollection = new ActionCollection( m_input );

	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( resized() ) );
	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( coordsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( maxIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( paintIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( startValueChanged() ), SIGNAL( stateChanged() ) );
}

Feigenbaum::Document::~Document()
{
}

bool Feigenbaum::Document::openFile( QString fileName )
{
	Handle<CalcInputData> input = m_fileaccess->open( fileName );

	if( ! input )
		return false;

	m_input = input;

	m_calc = new Calc( m_input, m_result );
	m_actioncollection = new ActionCollection( m_input );

	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( resized() ) );
	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( coordsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( maxIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( paintIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( startValueChanged() ), SIGNAL( stateChanged() ) );

	return true;
}

bool Feigenbaum::Document::saveFile( QString fileName )
{
	return m_fileaccess->save( fileName, m_input );
}

bool Feigenbaum::Document::exportBitmap( QString fileName )
{
	QFileInfo fi( fileName );
	if( fi.extension().lower() == "png" )
		return m_result->pixmap().save( fileName, "PNG" );
	else
		return m_result->pixmap().save( fileName, "BMP" );
}

QString Feigenbaum::Document::fileName() const
{
	return m_fileaccess->fileName();
}

void Feigenbaum::Document::printBitmap( const QPrinter* printer )
{
	m_result->printPixmap( printer );
}

void Feigenbaum::Document::setPicSize( const IntSize& size )
{
	if( size == m_input->picSize() ) return;

	m_actioncollection->add( new PicSizeAction( size, m_input->picSize() ) );
	m_input->setPicSize( size );
}

const IntSize& Feigenbaum::Document::picSize() const
{
	return m_input->picSize();
}

void Feigenbaum::Document::setCoords( const DoubleRect& coords )
{
	if( coords.width() == 0 || coords.height() == 0 ) return;

	m_actioncollection->add( new CoordsAction( coords, m_input->coords() ) );
	m_input->setCoords( coords );
}

const DoubleRect& Feigenbaum::Document::coords() const
{
	return m_input->coords();
}

void Feigenbaum::Document::startCalc()
{
	m_result = new CalcResult();
	m_calc = new Calc( m_input, m_result );

	connect( m_result.get(), SIGNAL( update( const QRect& ) ), SIGNAL( rectPainted( const QRect& ) ) );
	connect( m_calc.get(), SIGNAL( updated( const QRect& ) ), SIGNAL( rectPainted( const QRect& ) ) );
	connect( m_calc.get(), SIGNAL( started() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( finished() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( stopped() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( started() ), SIGNAL( startedCalc() ) );
	connect( m_calc.get(), SIGNAL( finished() ), SIGNAL( stoppedCalc() ) );
	connect( m_calc.get(), SIGNAL( stopped() ), SIGNAL( stoppedCalc() ) );

	m_calc->start();
}

void Feigenbaum::Document::stopCalc()
{
	m_calc->stop();
}

bool Feigenbaum::Document::calcing() const
{
	return m_calc->calcing();
}

void Feigenbaum::Document::undo()
{
	m_actioncollection->undo();
}

void Feigenbaum::Document::redo()
{
	m_actioncollection->redo();
}

bool Feigenbaum::Document::can_undo() const
{
	return m_actioncollection->can_undo();
}

bool Feigenbaum::Document::can_redo() const
{
	return m_actioncollection->can_redo();
}

QString Feigenbaum::Document::undo_descr() const
{
	return m_actioncollection->undo_description();
}

QString Feigenbaum::Document::redo_descr() const
{
	return m_actioncollection->redo_description();
}

QPixmap& Feigenbaum::Document::pixmap()
{
	return m_result->pixmap();
}

void Feigenbaum::Document::showProperties( QWidget* parent )
{
	PropDialog propdlg( parent );

	propdlg.setMaxIterations( m_input->maxIterations() );
	propdlg.setPaintIterations( m_input->paintIterations() );
	propdlg.setStartValue( m_input->startValue() );
	propdlg.setCoords( m_input->coords() );

	if( propdlg.exec() == QDialog::Accepted )
	{
		if( propdlg.maxIterations() != m_input->maxIterations() )
		{
			m_actioncollection->add( new MaxIterationsAction( propdlg.maxIterations(), m_input->maxIterations() ) );
			m_input->setMaxIterations( propdlg.maxIterations() );
		}

		if( propdlg.paintIterations() != m_input->paintIterations() )
		{
			m_actioncollection->add( new PaintIterationsAction( propdlg.paintIterations(), m_input->paintIterations() ) );
			m_input->setPaintIterations( propdlg.paintIterations() );
		}

		if( propdlg.coords() != m_input->coords() )
		{
			m_actioncollection->add( new CoordsAction( propdlg.coords(), m_input->coords() ) );
			m_input->setCoords( propdlg.coords() );
		}

		if( propdlg.startValue() != m_input->startValue() )
		{
			m_actioncollection->add( new StartValueAction( propdlg.startValue(), m_input->startValue() ) );
			m_input->setStartValue( propdlg.startValue() );
		}
	}
}

