/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "mandelbrot.h"

Mandelbrot::Document::Document()
	: DocInterface()
{
	m_input = new CalcInputData();
	m_result = new CalcResult();
	m_calc = new Calc( m_input, m_result );
	m_fileaccess = new FileAccess();
	m_actioncollection = new ActionCollection( m_input );

	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( resized() ) );
	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( coordsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( maxIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( exponentChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( colorFadeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( optModeChanged() ), SIGNAL( stateChanged() ) );
}

Mandelbrot::Document::~Document()
{
	if( m_calc->calcing() )
	{
		m_calc->stop();
		m_calc->wait();
	}
}

bool Mandelbrot::Document::openFile( QString fileName )
{
	Handle<CalcInputData> input = m_fileaccess->open( fileName );

	if( ! input )
		return false;

	m_input = input;

	m_calc = new Calc( m_input, m_result );
	m_actioncollection = new ActionCollection( m_input );

	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( resized() ) );
	connect( m_input.get(), SIGNAL( picSizeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( coordsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( maxIterationsChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( exponentChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( colorFadeChanged() ), SIGNAL( stateChanged() ) );
	connect( m_input.get(), SIGNAL( optModeChanged() ), SIGNAL( stateChanged() ) );

	return true;
}

bool Mandelbrot::Document::saveFile( QString fileName )
{
	return m_fileaccess->save( fileName, m_input );
}

bool Mandelbrot::Document::exportBitmap( QString fileName )
{
	return m_result->exportBitmap( fileName );
}

QString Mandelbrot::Document::fileName() const
{
	return m_fileaccess->fileName();
}

void Mandelbrot::Document::printBitmap( const QPrinter* printer )
{
	m_result->printPixmap( printer );
}

void Mandelbrot::Document::setPicSize( const IntSize& size )
{
	if( size == m_input->picSize() ) return;

	m_actioncollection->add( new PicSizeAction( size, m_input->picSize() ) );
	m_input->setPicSize( size );
}

const IntSize& Mandelbrot::Document::picSize() const
{
	return m_input->picSize();
}

void Mandelbrot::Document::setCoords( const DoubleRect& coords )
{
	if( coords.width() == 0 || coords.height() == 0 ) return;

	m_actioncollection->add( new CoordsAction( coords, m_input->coords() ) );
	m_input->setCoords( coords );
}

const DoubleRect& Mandelbrot::Document::coords() const
{
	return m_input->coords();
}

void Mandelbrot::Document::startCalc()
{
	m_result = new CalcResult();
	m_calc = new Calc( m_input, m_result );

	connect( m_calc.get(), SIGNAL( rectPainted( const IntRect& ) ), m_result.get(), SLOT( rectPainted( const IntRect& ) ) );
	connect( m_result.get(), SIGNAL( pixmapChanged( const QRect& ) ), SIGNAL( rectPainted( const QRect& ) ) );
	connect( m_calc.get(), SIGNAL( started() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( finished() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( stopped() ), SIGNAL( stateChanged() ) );
	connect( m_calc.get(), SIGNAL( started() ), SIGNAL( startedCalc() ) );
	connect( m_calc.get(), SIGNAL( finished() ), SIGNAL( stoppedCalc() ) );
	connect( m_calc.get(), SIGNAL( stopped() ), SIGNAL( stoppedCalc() ) );

	m_calc->start();
}

void Mandelbrot::Document::stopCalc()
{
	m_calc->stop();
}

bool Mandelbrot::Document::calcing() const
{
	return m_calc->calcing();
}

void Mandelbrot::Document::undo()
{
	m_actioncollection->undo();
}

void Mandelbrot::Document::redo()
{
	m_actioncollection->redo();
}

bool Mandelbrot::Document::can_undo() const
{
	return m_actioncollection->can_undo();
}

bool Mandelbrot::Document::can_redo() const
{
	return m_actioncollection->can_redo();
}

QString Mandelbrot::Document::undo_descr() const
{
	return m_actioncollection->undo_description();
}

QString Mandelbrot::Document::redo_descr() const
{
	return m_actioncollection->redo_description();
}

QPixmap& Mandelbrot::Document::pixmap()
{
	return m_result->pixmap();
}

void Mandelbrot::Document::showProperties( QWidget* parent )
{
	PropDialog propdlg( parent );

	propdlg.setMaxIterations( m_input->maxIterations() );
	propdlg.setExponent( m_input->exponent() );
	propdlg.setOptimization( m_input->optMode() );
	propdlg.setCoords( m_input->coords() );
	propdlg.setColorFade( m_input->colorFade() );

	if( propdlg.exec() == QDialog::Accepted )
	{
		if( propdlg.maxIterations() != m_input->maxIterations() )
		{
			m_actioncollection->add( new MaxIterationsAction( propdlg.maxIterations(), m_input->maxIterations() ) );
			m_input->setMaxIterations( propdlg.maxIterations() );
		}

		if( propdlg.exponent() != m_input->exponent() )
		{
			m_actioncollection->add( new ExponentAction( propdlg.exponent(), m_input->exponent() ) );
			m_input->setExponent( propdlg.exponent() );
		}

		if( propdlg.optimization() != m_input->optMode() )
		{
			m_actioncollection->add( new OptimizationAction( propdlg.optimization(), m_input->optMode() ) );
			m_input->setOptMode( propdlg.optimization() );
		}

		if( propdlg.coords() != m_input->coords() )
		{
			m_actioncollection->add( new CoordsAction( propdlg.coords(), m_input->coords() ) );
			m_input->setCoords( propdlg.coords() );
		}

		if( propdlg.colorFade() != m_input->colorFade() )
		{
			m_actioncollection->add( new ColorFadeAction( propdlg.colorFade(), m_input->colorFade() ) );
			m_input->setColorFade( propdlg.colorFade() );
		}
	}
}

