/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "mandelbrot_fileaccess.h"

#include <qdatastream.h>
#include <qfile.h>

Mandelbrot::FileAccess::FileAccess()
	: QObject()
{
}

Mandelbrot::FileAccess::~FileAccess()
{
}

Handle<Mandelbrot::CalcInputData> Mandelbrot::FileAccess::open( QString fileName )
{
	DoubleRect coords;
	unsigned int maxIterations;
	double exponent;
	ColorFade colorFade;
	CalcInputData::Optimization optMode;
	IntSize picSize;

	QFile file( fileName );
	if( ! file.open( IO_ReadOnly ) ) return 0;

	QDataStream ds( &file );

	// read signature to be sure we have a ChaosExplorer file
	char c_str[14];
	ds.readRawBytes( c_str, 14 );
	QString str( c_str );
	if( str != "CHAOSEXPLORER" ) return 0;

	// read file format version number (not used yet)
	int ver_maj, ver_min, ver_sub;
	ds >> ver_maj >> ver_min >> ver_sub;

	// read doc type
	int doc_type;
	ds >> doc_type;
	if( doc_type != 0 ) return 0; // only 0 corresponds to mandelbrot

	// read "the rest"
	ds >> coords.tl.x >> coords.tl.y;
	ds >> coords.br.x >> coords.br.y;
	ds >> maxIterations;
	ds >> exponent;
	
	unsigned int num_colors;
	ds >> num_colors;
	for( ColorFade::size_type i = 0; i < num_colors; i++ )
	{
		ColorStep cs;
		ds >> cs.red >> cs.green >> cs.blue >> cs.pos;
		if( colorFade.size() - 1 < i )
			colorFade.add( cs );
		else
			colorFade[i] = cs;
	}

	int dummy;
	ds >> dummy;
	optMode = static_cast<CalcInputData::Optimization>( dummy );
	ds >> picSize.width >> picSize.height;

	file.close();

	m_fileName = fileName;

	CalcInputData* cid = new CalcInputData();
	cid->setMaxIterations( maxIterations );
	cid->setExponent( exponent );
	cid->setOptMode( optMode );
	cid->setCoords( coords );
	cid->setColorFade( colorFade );
	cid->setPicSize( picSize );

	return cid;
}

bool Mandelbrot::FileAccess::save( QString fileName, Handle<CalcInputData> input )
{
	if( fileName.isEmpty() )
	{
		if( m_fileName.isEmpty() )
			return false;
		else
			fileName = m_fileName;
	}

	QFile file( fileName );
	if( ! file.open( IO_WriteOnly ) ) return false;

	QDataStream ds( &file );
	ds.writeRawBytes( "CHAOSEXPLORER\0", 14 ); // file type signature
	ds << 1 << 0 << 0; // file format version
	ds << 0; // type mandelbrot
	ds << input->coords().tl.x << input->coords().tl.y;
	ds << input->coords().br.x << input->coords().br.y;
	ds << input->maxIterations();
	ds << input->exponent();
	ds << input->colorFade().size();

	for( ColorFade::size_type i = 0; i < input->colorFade().size(); i++ )
	{
		ColorStep& cs = input->colorFade()[i];
		ds << cs.red << cs.green << cs.blue << cs.pos;
	}

	ds << input->optMode();
	ds << input->picSize().width << input->picSize().height;

	file.close();

	m_fileName = fileName;

	return true;
}

const QString& Mandelbrot::FileAccess::fileName() const
{
	return m_fileName;
}

bool Mandelbrot::FileAccess::unsaved() const
{
	return m_unsaved;
}

