/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "mandelbrot_propdialog.h"

Mandelbrot::PropDialog::PropDialog( QWidget* parent, const char* name )
	: QDialog( parent, name, true )
{

	setCaption( "Mandelbrot-Einstellungen" );

	QVBoxLayout* vbox = new QVBoxLayout( this, 10 );

	QTabWidget* tabwidget = new QTabWidget( this );
	vbox->addWidget( tabwidget );

	// create tab 1

	QWidget* calc = new QWidget( this );

	QVBoxLayout* calc_vbox = new QVBoxLayout( calc, 15 );

	QGridLayout* calc_grid = new QGridLayout( calc_vbox, 2, 2, 15 );
	m_max_iterations = new QSpinBox( 1, 10000, 1, calc );
	m_exponent = new QLineEdit( calc );
	m_exponent->setValidator( new QDoubleValidator( m_exponent ) );
	calc_grid->addWidget( new QLabel( "Iterationstiefe:", calc ), 0, 0 );
	calc_grid->addWidget( m_max_iterations, 0, 1 );
	calc_grid->addWidget( new QLabel( "Exponent:", calc ), 1, 0 );
	calc_grid->addWidget( m_exponent, 1, 1 );

	QVGroupBox* opt_gb = new QVGroupBox( "Optimierung", calc );
	calc_vbox->addWidget( opt_gb );
	m_opt = new QComboBox( false, opt_gb );
	m_opt->insertItem( "Keine" );
	m_opt->insertItem( "Interleave" );
	m_opt->insertItem( "Kstchen-Methode" );
	m_opt->insertItem( "Boundary Tracing" );
	connect( m_opt, SIGNAL( activated( int ) ), SLOT( optActivated( int ) ) );
	m_opt_descr = new QLabel( opt_gb );
	m_opt_descr->setAlignment( Qt::WordBreak );

	calc_vbox->addStretch();

	// create tab 2

	QWidget* coord = new QWidget( this );

	QVBoxLayout* coord_vbox = new QVBoxLayout( coord, 15 );

	QGroupBox* top_left = new QGroupBox( "Links oben", coord );
	coord_vbox->addWidget( top_left );
	QGridLayout* top_left_grid = new QGridLayout( top_left, 2, 2, 15 );
	m_x1 = new QLineEdit( top_left );
	m_x1->setValidator( new QDoubleValidator( m_x1 ) );
	m_y1 = new QLineEdit( top_left );
	m_y1->setValidator( new QDoubleValidator( m_y1 ) );
	top_left_grid->addWidget( new QLabel( "X:", top_left ), 0, 0 );
	top_left_grid->addWidget( m_x1, 0, 1 );
	top_left_grid->addWidget( new QLabel( "Y:", top_left ), 1, 0 );
	top_left_grid->addWidget( m_y1, 1, 1 );

	QGroupBox* bottom_right = new QGroupBox( "Rechts unten", coord );
	coord_vbox->addWidget( bottom_right );
	QGridLayout* bottom_right_grid = new QGridLayout( bottom_right, 2, 2, 15 );
	m_x2 = new QLineEdit( bottom_right );
	m_x2->setValidator( new QDoubleValidator( m_x2 ) );
	m_y2 = new QLineEdit( bottom_right );
	m_y2->setValidator( new QDoubleValidator( m_y2 ) );
	bottom_right_grid->addWidget( new QLabel( "X:", bottom_right ), 0, 0 );
	bottom_right_grid->addWidget( m_x2, 0, 1 );
	bottom_right_grid->addWidget( new QLabel( "Y:", bottom_right ), 1, 0 );
	bottom_right_grid->addWidget( m_y2, 1, 1 );

	coord_vbox->addStretch();

	// create tab 3

	QWidget* color = new QWidget( this );

	QVBoxLayout* color_vbox = new QVBoxLayout( color, 15 );

	m_fade = new ColorFadeWidget( color );
	color_vbox->addWidget( m_fade );

	// add tabs

	tabwidget->addTab( calc, "Berechnung" );
	tabwidget->addTab( coord, "Koordinaten" );
	tabwidget->addTab( color, "Farbpalette" );

	// add "ok"/"cancel" buttons

	QHBoxLayout* hbox = new QHBoxLayout( vbox, 10 );

	QPushButton* ok = new QPushButton( "&OK", this );
	QPushButton* cancel = new QPushButton( "Ab&brechen", this );
	connect( ok, SIGNAL( clicked() ), SLOT( accept() ) );
	connect( cancel, SIGNAL( clicked() ), SLOT( reject() ) );
	hbox->addStretch();
	hbox->addWidget( ok );
	hbox->addWidget( cancel );

}

Mandelbrot::PropDialog::~PropDialog()
{
}

void Mandelbrot::PropDialog::setMaxIterations( int iterations )
{
	m_max_iterations->setValue( iterations );
}

int Mandelbrot::PropDialog::maxIterations() const
{
	return m_max_iterations->value();
}

void Mandelbrot::PropDialog::setExponent( double exponent )
{
	m_exponent->setText( QString( "%1" ).arg( exponent ) );
}

double Mandelbrot::PropDialog::exponent() const
{
	double new_exponent;
	QTextStream stream( m_exponent->text(), IO_ReadOnly );
	stream >> new_exponent;
	if( new_exponent < 1.0 ) new_exponent = 1.0;
	return new_exponent;
}

void Mandelbrot::PropDialog::setOptimization( Mandelbrot::CalcInputData::Optimization opt )
{
	m_opt->setCurrentItem( opt );
	optActivated( opt );
}

Mandelbrot::CalcInputData::Optimization Mandelbrot::PropDialog::optimization() const
{
	return static_cast<CalcInputData::Optimization>( m_opt->currentItem() );
}

void Mandelbrot::PropDialog::setCoords( const DoubleRect& coords )
{
	m_x1->setText( QString( "%1" ).arg( coords.tl.x, 0, 'f', 16 ) );
	m_y1->setText( QString( "%1" ).arg( coords.tl.y, 0, 'f', 16 ) );
	m_x2->setText( QString( "%1" ).arg( coords.br.x, 0, 'f', 16 ) );
	m_y2->setText( QString( "%1" ).arg( coords.br.y, 0, 'f', 16 ) );
}

DoubleRect Mandelbrot::PropDialog::coords() const
{
	double x1, y1, x2, y2;
	QTextStream* stream;
	stream = new QTextStream( m_x1->text(), IO_ReadOnly );
	*stream >> x1;
	delete stream;
	stream = new QTextStream( m_y1->text(), IO_ReadOnly );
	*stream >> y1;
	delete stream;
	stream = new QTextStream( m_x2->text(), IO_ReadOnly );
	*stream >> x2;
	delete stream;
	stream = new QTextStream( m_y2->text(), IO_ReadOnly );
	*stream >> y2;
	delete stream;
	return DoubleRect( x1, y1, x2, y2 );
}

void Mandelbrot::PropDialog::setColorFade( const ColorFade& colorfade )
{
	m_fade->setColorFade( colorfade );
}

ColorFade Mandelbrot::PropDialog::colorFade() const
{
	return m_fade->colorFade();
}

void Mandelbrot::PropDialog::optActivated( int index )
{
	const char* descr[] =
	{
		"Berechnung jedes Punktes\n-genau\n-langsam",
		"berspringen gleichfarbiger Punkte einer Zeile\n-nutzt Symmetrie\n-ungenau\n-schnell",
		"Einteilung in einfarbige Rechtecke\n-evntl. ungenau\n-zgig bis mittlerer Iterationszahl",
		"Abtasten der Farbgrenzen\n-nutzt Symmetrie\n-genau\n-schnell"
	};

	m_opt_descr->setText( descr[ index ] );
}

